/*
 * arand.cpp
 *
 *  Created on: 26. 3. 2014
 *	  Author: Tomas Pecka
 */

#include <tclap/CmdLine.h>

#include <automaton/FSM/NFA.h>
#include <regexp/unbounded/UnboundedRegExp.h>
#include <factory/XmlDataFactory.hpp>
#include "automaton/generate/RandomAutomatonFactory.h"
#include "regexp/generate/RandomRegExpFactory.h"

int main(int argc, char* argv[]) {
	try {
		TCLAP::CmdLine cmd("Random generator binary", ' ', "0.01");

		std::vector<std::string> allowed;
		allowed.push_back("FSM");
		allowed.push_back("RE");
		TCLAP::ValuesConstraint<std::string> allowedVals( allowed );

		TCLAP::ValueArg<std::string> type(	"t",	"type",		"Type of generated structure",		true,	"FSM",	&allowedVals);
		cmd.add( type );

		TCLAP::ValueArg<int> alphabetSize(	"a",	"alphabetSize",	"Alphabet size",			false,	3,	"integer");
		cmd.add( alphabetSize );

		TCLAP::ValueArg<double> density(	"d",	"density",	"Automaton's transition density",	false,	10,	"double");
		cmd.add( density );

		TCLAP::ValueArg<int> nodes(		"n",	"nodes",	"Number of automaton's nodes",		false,	5,	"integer");
		cmd.add( nodes );

		TCLAP::ValueArg<int> leafNodes(		"l",	"leafNodes",	"Number of regexp's leaf nodes",	false,	5,	"integer");
		cmd.add( leafNodes );

		TCLAP::ValueArg<int> depth(		"e",	"depth",	"Depth of the regexp",			false,	5,	"integer");
		cmd.add( depth );

		cmd.parse(argc,argv);

		if(!type.isSet()) throw exception::AlibException("Type is not defined.");
		if( type.getValue() == "FSM" ) {

			if(!density.isSet()) throw exception::AlibException("Density is not defined.");
			if( density.getValue() < 1 || density.getValue() > 100 )
			{
				//TODO: floating point arithmetic
				throw exception::AlibException("You must specify density as a number between 1 and 100.");
				return 1;
			}

			if(!nodes.isSet()) throw exception::AlibException("Number of nodes is not defined.");
			if(!alphabetSize.isSet()) throw exception::AlibException("Alphabet size is not defined.");

			automaton::NFA res = automaton::generate::RandomAutomatonFactory::generateNFA( nodes.getValue(), alphabetSize.getValue(), density.getValue() );
			alib::XmlDataFactory::toStdout(res);
		} else if( type.getValue() == "RE" ) {

			if(!depth.isSet()) throw exception::AlibException("Depth is not defined.");

			if(!leafNodes.isSet()) throw exception::AlibException("Number of leaf nodes is not defined.");
			if(!alphabetSize.isSet()) throw exception::AlibException("Alphabet size is not defined.");

			regexp::UnboundedRegExp res = regexp::generate::RandomRegExpFactory::generateUnboundedRegExp(leafNodes.getValue(), depth.getValue(), alphabetSize.getValue() );
			alib::XmlDataFactory::toStdout(res);
		} else {
			throw exception::AlibException("Invalid type.");
		}

		return 0;
	} catch( const exception::AlibException & exception ) {
		alib::XmlDataFactory::toStdout( exception );
		return 1;
	} catch(const TCLAP::ArgException& exception) {
		std::cout << exception.error() << std::endl;
		return 2;
	} catch (const std::exception& exception) {
		std::cerr << "Exception caught: " << exception.what() << std::endl;
		return 3;
	} catch(...) {
		std::cerr << "Unknown exception caught." << std::endl;
		return 127;
	}
}
